function [hm_MDS , myRect , Ie] = CoETM ( Template , I ,K ,Nd)
% ------------------------------------------------------------------------------------------
% CoETM : maps the input image (I) and the template to an embedded image such that the Euclidean
% distance between pixel values in the embedded space resembles the co-occurrence statistics 
% in the original space. Then, compute SSD error measere between the
% embedded template and each sliding window in the embedded image.
%
% Syntax: 
%   [hm , myRect] = CoETM ( Template , I ,K ,Nd)
%
% Inputs: 
%   Template: a template from a source image
%   I: query Image of size n x m x d
%   K: number of unique values (quantized clusters). 
%   Nd: embedded space dimention

% Output:
%   hm: Likelihood map 
%   myRect: The best result in CoTM measure sense (max hm)
%   Iembedded : embedded image of size n x m x Nd 
%
% ------------------------------------------------------------------------------------------


    if size(I,3)==1 %Grayscale        
         right.idx = double(I)+1;
         T.idx     = double(Template)+1;
         right.sz    = size( right.idx );
         T.sz        = size( T.idx );         
    else
        right.rgb = I;
        T.rgb     = Template;
        % Transform to Lab
        right.lab   = rgb2lab( right.rgb );     right.sz    = size( right.rgb );
        T.lab       = rgb2lab( T.rgb );         T.sz        = size( T.rgb );
    
       % option1 
        f =cat(1, reshape( right.lab, [], 3 ), reshape( T.lab, [], 3 ) ); 
        [q, ~] = kmeans(f, K , 'maxIter' , 300); 
        right.idx = reshape((q(1:right.sz(1)*right.sz(2))),size(right.lab,1),size(right.lab,2)); 
        T.idx     = reshape((q(right.sz(1)*right.sz(2)+1:end)),size(T.lab,1),size(T.lab,2));       

%       % option2
%       scale =1;
%       f = right.lab;
%       f = f(1:scale:end,1:scale:end,:);        
%       f = cat(1, reshape(f,[],3), reshape( T.lab, [], 3 ) ); 
%       [~, cc] = kmeans(f, K ); 
% 
%       % Qunatize the image according cc: 
%       right.idx = zeros( right.sz(1:2) );
%       T.idx = zeros( T.sz(1:2) );
%       right.mv    = inf( size( right.idx ) );
%       T.mv        = inf( size( T.idx ) ) ;
%            
%       for k = 1:K              
%            [right.mv, minIdx ] = min( cat( 3, right.mv, sum( ( right.lab - repmat( permute( cc( k, : ), [3, 1, 2] ), [right.sz(1:2), 1 ] ) ).^2, 3 ) ), [], 3 );
%            right.idx( minIdx == 2 ) = k;               
%            [T.mv, minIdx ] = min( cat( 3, T.mv, sum( ( T.lab - repmat( permute( cc( k, : ), [3, 1, 2] ), [T.sz(1:2), 1 ] ) ).^2, 3 ) ), [], 3 );
%            T.idx( minIdx == 2 ) = k;       
%       end    
    end   

    % Co-occurrence:
    sigma = sqrt((T.sz(1:2)-1)/2);
    fg = fspecial( 'gaussian', 2*ceil(2*(sigma))+1, min(sigma) ); %rotal-change
    %HARD
    C = collectJProb( uint8( right.idx - 1 ), ones( size( right.idx ) ), K , fg ,0 )+...
        collectJProb( uint8( T.idx - 1 ), ones( size( T.idx ) ), K , fg , 1 );
    % Probability function normalization
    C = C + C.';
    C = C/sum(C(:)); 

    % Similarity matrix normalization  
    pab_n = zeros(K);
    for k = 1:K
        pab_n(k, k) = 1;
        for ik = (k+1):K
            pab_n(k, ik) = C(k, ik)/(sqrt(C(k, k) * C(ik, ik) ) + eps); %sqrt(eps + C(k, k) * C(ik, ik) );
            pab_n(ik, k) = pab_n(k, ik);
        end
    end
    pab_n(pab_n>1)=1;pab_n(pab_n==0)=eps;
    M_n=pab_n;
          
    [rep] = getEmbeddedRepForPMI( M_n , Nd);  
    % Create embedded images
    right.MDS = reshape( rep( right.idx(:) , : ), size(right.idx, 1), size(right.idx, 2), [] );
    T.MDS = reshape( rep( T.idx(:) , : ), size(T.idx, 1), size(T.idx, 2), [] );
    
    % SSD lookup:    
    hm_MDS      = nan( right.sz(1:2) );       
    for wr =  ceil(T.sz(1)/2 ):( right.sz(1) - ceil(T.sz(1)/2 ) )
        for wc =  ceil(T.sz(2)/2 ):( right.sz(2)- ceil(T.sz(2)/2 ) ) 
            temp = right.MDS( ...
                ( wr - ceil( T.sz(1)/2 ) + 1): ( wr - ceil( T.sz(1)/2 ) + T.sz(1)), ...
                ( wc - ceil( T.sz(2)/2 ) + 1): ( wc - ceil( T.sz(2)/2 ) + T.sz(2)) , :);
            hm_MDS ( wr, wc ) = norm(temp(:)-T.MDS(:),2);    
        end
    end
         
    hm_MDS      = norm_map(hm_MDS,'min');
    [r_MDS, c_MDS, ~]  = find( hm_MDS  == max( hm_MDS(:) ) );
    myRect = [c_MDS - ceil(T.sz(2)/2), r_MDS - ceil( T.sz(1)/2 ), T.sz(2), T.sz(1)];
    Ie = right.MDS;
end