function [hm , myRect] = CoTM ( Template , I ,K)
% ------------------------------------------------------------------------------------------
% CoTM: compute CoTM error measure with each sliding window in the input image I using color pixels 
%
% Syntax: 
%   [hm , myRect] = CoTM ( Template , I ,K)
%
% Inputs: 
%   Template: a template from a source image
%   I: query Image
%   K: number of unique values (quantized clusters). 
%
% Output:
%   hm: Likelihood map 
%   myRect: The best result in CoTM measure sense (max hm)
%
% ------------------------------------------------------------------------------------------


    if (size( I,3)==3 && ((sum(sum(I(:,:,1)))-sum(sum(I(:,:,2))) )==0 && (sum(sum(I(:,:,2)))-sum(sum(I(:,:,3))))==0 ) )       %Grayscale        
        I = rgb2gray(I);
    end
    
    if (size( I,3)==1) 
     
         right.idx = double(I)+1;
         T.idx     = double(Template)+1;
         right.sz    = size( right.idx );
         T.sz        = size( T.idx );         
    else
        right.rgb = I;
        T.rgb     = Template;
        % Transform to Lab
        right.lab   = rgb2lab( right.rgb );     right.sz    = size( right.rgb );
        T.lab       = rgb2lab( T.rgb );         T.sz        = size( T.rgb );
    
       % option1 
        f =cat(1, reshape( right.lab, [], 3 ), reshape( T.lab, [], 3 ) ); 
        [q, ~] = kmeans(f, K , 'maxIter' , 300); 
        right.idx = reshape((q(1:right.sz(1)*right.sz(2))),size(right.lab,1),size(right.lab,2)); 
        T.idx     = reshape((q(right.sz(1)*right.sz(2)+1:end)),size(T.lab,1),size(T.lab,2));       

%       % option2
%       scale =1;
%       f = right.lab;
%       f = f(1:scale:end,1:scale:end,:);        
%       f = cat(1, reshape(f,[],3), reshape( T.lab, [], 3 ) ); 
%       [~, cc] = kmeans(f, K ); 
% 
%       % Qunatize the image according cc: 
%       right.idx = zeros( right.sz(1:2) );
%       T.idx = zeros( T.sz(1:2) );
%       right.mv    = inf( size( right.idx ) );
%       T.mv        = inf( size( T.idx ) ) ;
%            
%       for k = 1:K              
%            [right.mv, minIdx ] = min( cat( 3, right.mv, sum( ( right.lab - repmat( permute( cc( k, : ), [3, 1, 2] ), [right.sz(1:2), 1 ] ) ).^2, 3 ) ), [], 3 );
%            right.idx( minIdx == 2 ) = k;               
%            [T.mv, minIdx ] = min( cat( 3, T.mv, sum( ( T.lab - repmat( permute( cc( k, : ), [3, 1, 2] ), [T.sz(1:2), 1 ] ) ).^2, 3 ) ), [], 3 );
%            T.idx( minIdx == 2 ) = k;       
%       end    
    end   
  
    % Co-occurrence:
    sigma = sqrt((T.sz(1:2)-1)/2);
    fg = fspecial( 'gaussian', 2*ceil(2*(sigma))+1, min(sigma) ); 
    C = collectJProb( uint8( right.idx - 1 ), ones( size( right.idx ) ), K, fg , 0 )+...
        collectJProb( uint8( T.idx - 1 ), ones( size( T.idx ) ), K, fg, 1 );
    % Probability function normalization
    C = C + C.'; 
    C = C/sum(C(:)); 
    M =  C./ ( sum( C ).' * sum(C) + eps );

    % Co-occurrence lookup:
    hm  = zeros( right.sz(1:2) );
    id_board = T.idx(:) - 1;
    id_board = K * id_board;   
    for wr =  1:(right.sz(1)-T.sz(1)+1) 
        for wc =  1: (right.sz(2)-T.sz(2)+1) 
            ids = right.idx(wr : wr + T.sz(1)-1, wc : wc + T.sz(2)-1 );                  
            ids = ids(:) + id_board ;
            hm( wr, wc )  = sum(  M(ids));  
        end
    end

    [r, c, ~] = find( hm == max( hm(:) ),1 ); 
    myRect = [c , r , T.sz(2), T.sz(1)];
    hm = MoveToCenter(hm, T.sz, 0);  %change hm for display:
end