function [hm , myRect] = CoTM_deep ( Template , I ,K , features)
% ------------------------------------------------------------------------------------------
% CoTM_deep: Compute CoTM error measure with each sliding window in the input image I 
% using deep features.  
%
% Syntax: 
%   [hm , myRect] = CoTM ( Template , I ,K , features)
%
% Inputs: 
%   Template: a template from a source image
%   I: query Image
%   K: number of unique values (quantized clusters). 
%   CNN features: features.net = imagenet-vgg-verydeep-19.mat 
%                 features.deep_feat_layers = [4,18]
%
% Output:
%   hm: Likelihood map 
%   myRect: The best result in CoTM measure sense (max hm)
% ------------------------------------------------------------------------------------------
    right.sz    = size( I );
    T.sz        = size( Template );

    % Extract deep features
    f = extract_features_CoTM(features.net, I, features.deep_feat_layers);
    f_t = extract_features_CoTM(features.net, Template , features.deep_feat_layers);         
    f = cat(1, reshape(f,[],size(f,3)), reshape( f_t , [], size(f_t,3) ) ); 
    
    % Option 1: Using KMEANS
    [q, ~] = kmeans(f, K );
    
%     % Option 2: Using PCA (fast)
%     f_t_rep = repmat(f_t , round(0.5*right.sz(1:2)./T.sz(1:2))); 
%     f_pca = cat(1, f , reshape( f_t_rep , [], size(f_t,3) ) );   
%     [cc, ~,~] = pca( f_pca ); 
%      cc = cc(:,1:8);     
%     rep = f*cc;
%     rep = sign(rep);
%     rep(rep==0)=1;
%     rep(rep<0)=0;
%     q = bi2de(rep)+1;
    
    right.idx = reshape((q(1:right.sz(1)*right.sz(2))),right.sz(1),right.sz(2)); 
    T.idx     = reshape((q(right.sz(1)*right.sz(2)+1:end)),T.sz(1),T.sz(2)); 

    % Co-occurrence:
    sigma = sqrt((T.sz(1:2)-1)/2);
    fg = fspecial( 'gaussian', 2*ceil(2*(sigma))+1, min(sigma) ); 
    C = collectJProb( uint8( right.idx - 1 ) , ones( size( right.idx ) ) , K , fg  ,0 )+...
        collectJProb( uint8( T.idx - 1 ), ones( size( T.idx ) ) , K , fg  , 1 );
    % Probability function normalization
    C = C + C.'; 
    C = C/sum(C(:)); 
    M =  C./ ( sum( C ).' * sum(C) + eps );
   
    % Co-occurrence lookup:
    hm  = zeros( right.sz(1:2) );
    id_board = T.idx(:) - 1;
    id_board = K * id_board;
    for wr =  1:(right.sz(1)-T.sz(1)+1) 
        for wc =  1: (right.sz(2)-T.sz(2)+1) 
            ids = right.idx(wr : wr + T.sz(1)-1, wc : wc + T.sz(2)-1 );           
            ids = ids(:) + id_board ;
            hm( wr, wc )  = sum( M(ids));  
        end
    end

    [r, c, ~] = find( hm == max( hm(:) ),1 );
    myRect = [c , r , T.sz(2), T.sz(1)];       
    hm = MoveToCenter(hm, T.sz, 0);  %change hm for display:
end