function JP = collectJProb( im2collect, mask2collect, nBins, f_cooc , biasFlag  )
% COLLECTPMI collects  P(a,b) / Pr(a)Pr(b) 
%
% Syntax: 
%   JP = collectJProb( im2collect, mask2collect, nBins, f_cooc )
%
% Inputs: 
%   im2collect: collect P(a,b) from this im2collect.
%   mask2collect: collect values only where mask is non zero.
%   nBins: number of unique values. 
%   f_cooc: used for spatial weighting. 
%
% Output:
%   JP: a matrix [nBinsXnBins] with the collected joint - probabilities.
%

    %% Default Parameters:
    if( isa( im2collect, 'uint8' ) ), im2collect = double( im2collect ); end
    if( nargin < 2 ), mask2collect = ones( size( im2collect ) ); end
    if( nargin < 3 ), nBins = 256; end
    if( nargin < 4 ), f_cooc = fspecial( 'gaussian', 15, sqrt(15) ); end


    %% Processing Parameters: 
    sz = size( im2collect ); 
    if( length(sz) == 2 ), sz = [sz, 1]; end
    JP = zeros( nBins, nBins, sz(3) );

    %Cancel bias of high frequenct pixel values  
    if (~biasFlag)
        f_cooc( ( size(f_cooc,1) + 1 )/2, ( size(f_cooc,2) + 1 )/2 ) = 0; 
        f_cooc = f_cooc./sum(f_cooc(:));
    end
    
    %% Process Per Channel:
    for iChannel = 1:sz(3)       
        % Slice image to level:    
        cref = im2collect( :, :, iChannel );
        pab = zeros( nBins, nBins );   
        for iLevel = 0:( nBins - 1 )
            w = conv2( double( cref == iLevel ) .* mask2collect( :, :, iChannel ), f_cooc, 'same' );                      
            pab( iLevel + 1, : ) = accumarray( cref(:) + 1 , w(:), [ nBins, 1 ] );    
        end       
        JP( :, :, iChannel ) = pab ./( sum( pab ).' * sum(pab) + eps );        
    end

end








