function [image_deep] = extract_features_CoTM(net, img, feat_layers)

    img = im2uint8(img);
    im_ = single(img) ; % note: 255 range
    im_ = im_ -repmat(net.meta.normalization.averageImage,size(im_,1),size(im_,2),1) ; 

    %im2patches
    patch_sz = [224,224];
    patch_stride = [100,100];
    szI = size(im_);
    n_patches = [ceil(szI(1)/patch_stride(1)),ceil(szI(2)/patch_stride(2))];
    patches = cell(n_patches);
    for i = 1:patch_stride(1):szI(1)
        for j=1:patch_stride(2):szI(2)
            ind_i = 1+(i-1)/patch_stride(1);
            ind_j = 1+(j-1)/patch_stride(2);
            temp = im_(i:min(szI(1),i+patch_sz(1)-1),j:min(szI(2),j+patch_sz(2)-1),:) ;
            padd_amount = patch_sz - [size(temp,1),size(temp,2)];
            patches{ind_i,ind_j} = padarray ( temp , [padd_amount,0],0,'post');
        end
    end

    patches_deep = cell(n_patches);

    for ind_i=1:n_patches(1)
        for ind_j=1:n_patches(2) 

            im_ = patches{ind_i,ind_j};
            % Run the CNN.
            res = vl_simplenn(net, im_) ;

            feat = [];
            feat_layers = sort(feat_layers,'ascend');
            for f = 1:length(feat_layers)
                tmp = res(feat_layers(f)).x;
                tmp = imresize(tmp, patch_sz,'method','bilinear');        
                %L2 norm:
                l2 = repmat(sqrt(max(sum(tmp.^2,3),1e-12)),1,1,size(tmp,3));
                tmp = tmp./l2;          
                feat = cat(3,feat,gather(tmp));
            end

            patches_deep{ind_i,ind_j} = feat;

        end
    end

    %patches2im
    image_deep = zeros([szI(1:2),320]);
    patch_hsz = floor((patch_sz-[1,1])./2);

    for r = 1:szI(1)     %w
        for c = 1:szI(2) %h

             %Find the best patch location in sense of : min  |(r,c)-(patch_hsz+patch_stride.*(w,h))|
             w = max(0,min(n_patches(1),round( ( r-1-patch_hsz(1) ) / patch_stride(1) ))); 
             h = max(0,min(n_patches(2),round( ( c-1-patch_hsz(2) ) / patch_stride(2) ))); 

             % Find row and column within patch
             pr = r-w*patch_stride(1);
             pc = c-h*patch_stride(2);

             image_deep(r,c,:) = patches_deep{w+1,h+1}(pr,pc,:);            
        end
    end
end