function [rep] = getEmbeddedRepForPMI( pmi , Nd)
% ------------------------------------------------------------------------------------------
%getEmbeddedRepForPMI: get co-occurrence embedded representation having pmi and indexed image  
%   
% Syntax:
%   [rep] = getEmbeddedRepForPMI( pmi , Nd )
%
% Input: 
%   pmi: pmi similarity matrix [kXk] (normalized in 0-1)
%
% Output: 
%   rep: representation matrix [kXNd] , a configuration of k points (rows) in Nd dimensions (columns). 
% ------------------------------------------------------------------------------------------


    % PMI similarity -> dissimilarity
    D = -log(pmi);             
    D = 0.5*(D + D.');    
    
    % MDS [classic] for initial guess
    [rep,~] = cmdscale(D); 
    N = size(rep,2);
    opts = statset('MaxIter',500);
    try
        [rep,~] = mdscale(D,min(Nd,N),'Options',opts);   
    catch
        [rep,~] = mdscale(D,min(Nd,N),'criterion','sstress','Options',opts);
    end

    % Optional: PCA on all data
    %   x = rep(img.idx(:),:);
    %   [p, ~] = pca( x ); 
    %   rep = rep*p;
        

end
