% ------------------------------------------------------------------------------------------
% Co-Occurrence Template Matching (CoTM) - DEMO
% Run CoTM on 105 template - image pairs created by Dekel et al. [5] from 35 annotated color
% video sequences of the OTB dataset [31]
%
% [5] T. Dekel, S. Oron, M. Rubinstein, S. Avidan, andW. T. Freeman. Best-buddies similarity for robust template matching.
% In 2015 IEEE Conference on Computer Vision and Pattern Recognition (CVPR), pages 20212029. IEEE, 2015.
% [31] Y. Wu, J. Lim, and M.-H. Yang. Object tracking benchmark. IEEE Transactions on Pattern Analysis and Machine Intelligence, 37(9):18341848, 2015.
% ------------------------------------------------------------------------------------------

clc;
close all; clear all;
path_in   = '.\Data';
addpath('Utils');
verbose = 1; %Display results
Kmeans = 256;    % K of kmeans 
%% Load Images and template: ---------------------------------------------------------------

pairNum = 1; %USER DEFINE (pairs 1-105)
[Iquery, Iref ,Template, rect,gtRect] = loadImageAndTemplate( pairNum, path_in);


%% Color pixels: ---------------------------------------------------------------------------

[HeatMap,CoTMrect] = CoTM (Template,Iquery,Kmeans);

%Compute intersection over union with ground truth
IoU_color =  calcRectNormalizedOverlap(CoTMrect,gtRect);  
fprintf( 'CoTM on color pixels - pair %03d: IoU = %0.4f \n', pairNum, IoU_color  );  

if (verbose)
    figure;
    subplot(2,2,1);imshow(Iref); title('Template'); 
    hold on; rectangle('Position', rect, 'EdgeColor', [0 1 0], 'Linewidth', 2);
    subplot(2,2,2);imshow(Iquery); title('Query Image');   
    hold on;rectangle('Position', gtRect, 'EdgeColor', [0 1 0], 'Linewidth', 2); 
    hold on;rectangle('Position', CoTMrect, 'EdgeColor', [1 0 0], 'Linewidth', 2);  
    subplot(2,2,3);imagesc(HeatMap);  
    hold off;
    suptitle('CoTM Results (Red) on color pixels:'); 
end

%% Deep features (using Matcovnet) ---------------------------------------------------------

% loading imagenet-vgg-verydeep-19.mat
% you can downlaod the net from http://www.vlfeat.org/matconvnet/pretrained/
% you can download matcovnet from http://www.vlfeat.org/matconvnet/

% Setup Matconvnet
addpath ('C:\matconvnet-1.0-beta24\matlab\');
run vl_compilenn;
run vl_setupnn;

% Load model: (update your path)
features.deep_feat_layers = [4,18];
features.net = load(fullfile('C:\Users\Raz\Dropbox\MasterCV\Tracking_Work\Deep\vgg_model', 'imagenet-vgg-verydeep-19.mat'));
features.net = vl_simplenn_tidy(features.net) ;
features.net.layers(max(features.deep_feat_layers)+1:end) = [];  % Remove top layers and use only low level features  

% Setup GPU
gpuN = gpuDeviceCount; 
if gpuN>0 
    gpuDevice(1);
    features.net = vl_simplenn_move(features.net, 'gpu'); 
end

[HeatMap,CoTMrect] = CoTM_deep (Template,Iquery,Kmeans,features);

%Compute intersection over union with ground truth
IoU_color =  calcRectNormalizedOverlap(CoTMrect,gtRect);  
fprintf( 'CoTM on deep features - pair %03d: IoU = %0.4f \n', pairNum, IoU_color  );  

if (verbose)
    figure;
    subplot(2,2,1);imshow(Iref); title('Template'); 
    hold on; rectangle('Position', rect, 'EdgeColor', [0 1 0], 'Linewidth', 2);
    subplot(2,2,2);imshow(Iquery); title('Query Image');   
    hold on;rectangle('Position', gtRect, 'EdgeColor', [0 1 0], 'Linewidth', 2); 
    hold on;rectangle('Position', CoTMrect, 'EdgeColor', [1 0 0], 'Linewidth', 2);  
    subplot(2,2,3);imagesc(HeatMap);  
    hold off;
    suptitle('CoTM Results (Red) on deep features:'); 
end

%% Co-Occurence Embedding (Using MDS) ------------------------------------------------------

D = 256; %USER DEFINE: required dimension of embedded space 
[HeatMap,CoETMrect,Iembedded] = CoETM (Template,Iquery,Kmeans,D);

%Compute intersection over union with ground truth bounding box
IoU_color =  calcRectNormalizedOverlap(CoETMrect,gtRect);  
fprintf( 'CoETM on %d dimensions - pair %03d: IoU = %0.4f \n', D, pairNum, IoU_color  );  


if (verbose)
    figure;
    subplot(2,2,1);imshow(Iref); title('Template'); 
    hold on; rectangle('Position', rect, 'EdgeColor', [0 1 0], 'Linewidth', 2);
    subplot(2,2,2);imshow(Iquery); title('Query Image');   
    hold on;rectangle('Position', gtRect, 'EdgeColor', [0 1 0], 'Linewidth', 2); 
    hold on;rectangle('Position', CoTMrect, 'EdgeColor', [1 0 0], 'Linewidth', 2);  
    subplot(2,2,3);imagesc(HeatMap); title('Likelihood Map'); 
    %Normalize for display   
    Iembedded_d = Iembedded(:,:,1:3); 
    Iembedded_d = Iembedded_d + abs(min(Iembedded_d(:))); 
    Iembedded_d = (Iembedded_d - min(Iembedded_d(:)))./ (max(Iembedded_d(:))-min(Iembedded_d(:))); 
    subplot(2,2,4);imagesc(Iembedded_d); title('Embedded Image');  
    hold off;
    suptitle(sprintf('CoETM Results (Red) on %d dimentions:',D)); 
end

